@@ -39,7 +39,7 @@ struct mtrand {
   uint32_t mt_buffer_[MT_LEN];
 };
 
-int compare_files (const char* tgt, const char *rec);
+int test_compare_files (const char* tgt, const char *rec);
 void mt_init(mtrand *mt, uint32_t seed);
 uint32_t mt_random (mtrand *mt);
 int test_setup (void);
@@ -118,7 +118,7 @@ mt_exp_rand (uint32_t mean, uint32_t max_value)
 
 #define MSG_IS(x) (stream->msg != NULL && strcmp ((x), stream->msg) == 0)
 
-static const usize_t TWO_MEGS_AND_DELTA = (2 << 20) + (1 << 10);
+static const usize_t TWO_MEGS_AND_DELTA = (3 << 20);
 static const usize_t ADDR_CACHE_ROUNDS = 10000;
 
 static const usize_t TEST_FILE_MEAN   = 16384;
@@ -369,7 +369,7 @@ test_make_inputs (xd3_stream *stream, xoff_t *ss_out, xoff_t *ts_out)
 }
 
 int
-compare_files (const char* tgt, const char *rec)
+test_compare_files (const char* tgt, const char *rec)
 {
   FILE *orig, *recons;
   static uint8_t obuf[TESTBUFSIZE], rbuf[TESTBUFSIZE];
@@ -1590,9 +1590,12 @@ test_streaming (xd3_stream *in_stream, uint8_t *encbuf, uint8_t *decbuf, uint8_t
   xd3_stream estream, dstream;
   int ret;
   usize_t i, delsize, decsize;
+  xd3_config cfg;
+  xd3_init_config (& cfg, in_stream->flags);
+  cfg.flags |= XD3_COMPLEVEL_6;
 
-  if ((ret = xd3_config_stream (& estream, NULL)) ||
-      (ret = xd3_config_stream (& dstream, NULL)))
+  if ((ret = xd3_config_stream (& estream, & cfg)) ||
+      (ret = xd3_config_stream (& dstream, & cfg)))
     {
       goto fail;
     }
@@ -1605,7 +1608,7 @@ test_streaming (xd3_stream *in_stream, uint8_t *encbuf, uint8_t *decbuf, uint8_t
 
       if ((ret = xd3_process_stream (1, & estream, xd3_encode_input, 0,
 				     encbuf, 1 << 20,
-				     delbuf, & delsize, 1 << 10)))
+				     delbuf, & delsize, 1 << 20)))
 	{
 	  in_stream->msg = estream.msg;
 	  goto fail;
@@ -1645,11 +1648,21 @@ static int
 test_compressed_stream_overflow (xd3_stream *stream, int ignore)
 {
   int ret;
+  int i;
   uint8_t *buf;
 
   if ((buf = (uint8_t*) malloc (TWO_MEGS_AND_DELTA)) == NULL) { return ENOMEM; }
 
   memset (buf, 0, TWO_MEGS_AND_DELTA);
+  for (i = 0; i < (2 << 20); i += 256) 
+    {
+      int j;
+      int off = mt_random(& static_mtrand) % 10;
+      for (j = 0; j < 256; j++) 
+	{
+	  buf[i + j] = j + off;
+	}
+    }
 
   /* Test overflow of a 32-bit file offset. */
   if (SIZEOF_XOFF_T == 4)
@@ -1670,8 +1683,14 @@ test_compressed_stream_overflow (xd3_stream *stream, int ignore)
     }
 
   /* Test transfer of exactly 32bits worth of data. */
-  if ((ret = test_streaming (stream, buf, buf + (1 << 20), buf + (2 << 20), 1 << 12))) { goto fail; }
-
+  if ((ret = test_streaming (stream, 
+			     buf, 
+			     buf + (1 << 20), 
+			     buf + (2 << 20), 
+			     1 << 12))) 
+    {
+      goto fail;
+    }
  fail:
   free (buf);
   return ret;
@@ -1752,7 +1771,7 @@ test_command_line_arguments (xd3_stream *stream, int ignore)
 	}
 
       /* Compare the target file. */
-      if ((ret = compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
+      if ((ret = test_compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
 	{
 	  return ret;
 	}
@@ -1783,12 +1802,12 @@ test_command_line_arguments (xd3_stream *stream, int ignore)
 	  return XD3_INTERNAL;
 	}
 
-      /* Also check that compare_files works.  The delta and original should
+      /* Also check that test_compare_files works.  The delta and original should
        * not be identical. */
-      if ((ret = compare_files (TEST_DELTA_FILE,
+      if ((ret = test_compare_files (TEST_DELTA_FILE,
 				TEST_TARGET_FILE)) == 0)
 	{
-	  stream->msg = "broken compare_files";
+	  stream->msg = "broken test_compare_files";
 	  return XD3_INTERNAL;
 	}
 
@@ -1985,7 +2004,7 @@ test_recode_command2 (xd3_stream *stream, int has_source,
     }
 
   /* Now compare. */
-  if ((ret = compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
+  if ((ret = test_compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
     {
       return ret;
     }
@@ -2070,7 +2089,7 @@ test_compressed_pipe (xd3_stream *stream, main_extcomp *ext, char* buf,
       return XD3_INTERNAL;
     }
 
-  if ((ret = compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
+  if ((ret = test_compare_files (TEST_TARGET_FILE, TEST_RECON_FILE)))
     {
       return XD3_INTERNAL;
     }
@@ -2200,7 +2219,7 @@ test_source_decompression (xd3_stream *stream, int ignore)
   snprintf_func (buf, TESTBUFSIZE, "%s -v -dq -R -s%s %s %s", program_name,
 	   TEST_SOURCE_FILE, TEST_DELTA_FILE, TEST_RECON_FILE);
   if ((ret = do_cmd (stream, buf))) { return ret; }
-  if ((ret = compare_files (TEST_COPY_FILE,
+  if ((ret = test_compare_files (TEST_COPY_FILE,
 			    TEST_RECON_FILE))) { return ret; }
 
   /* Decode the delta file with recompression, should get a compressed file
@@ -2211,7 +2230,7 @@ test_source_decompression (xd3_stream *stream, int ignore)
   snprintf_func (buf, TESTBUFSIZE, "%s %s < %s > %s", ext->decomp_cmdname, ext->decomp_options,
 	   TEST_RECON_FILE, TEST_RECON2_FILE);
   if ((ret = do_cmd (stream, buf))) { return ret; }
-  if ((ret = compare_files (TEST_COPY_FILE,
+  if ((ret = test_compare_files (TEST_COPY_FILE,
 			    TEST_RECON2_FILE))) { return ret; }
 
   /* Encode with decompression disabled */
@@ -2224,7 +2243,7 @@ test_source_decompression (xd3_stream *stream, int ignore)
   snprintf_func (buf, TESTBUFSIZE, "%s -d -D -vfq -s%s %s %s", program_name,
 	   TEST_SOURCE_FILE, TEST_DELTA_FILE, TEST_RECON_FILE);
   if ((ret = do_cmd (stream, buf))) { return ret; }
-  if ((ret = compare_files (TEST_TARGET_FILE,
+  if ((ret = test_compare_files (TEST_TARGET_FILE,
 			    TEST_RECON_FILE))) { return ret; }
 
   test_cleanup();
@@ -2584,7 +2603,8 @@ test_string_matching (xd3_stream *stream, int ignore)
 	    default: CHECK(0);
 	    }
 
-	  snprintf_func (rptr, rbuf+TESTBUFSIZE-rptr, "%d/%d", inst->pos, inst->size);
+	  snprintf_func (rptr, rbuf+TESTBUFSIZE-rptr, "%d/%d", 
+			 inst->pos, inst->size);
 	  rptr += strlen (rptr);
 
 	  if (inst->type == XD3_CPY)
@@ -2815,7 +2835,7 @@ xd3_selftest (void)
   DO_TEST (decompress_single_bit_error, 0, 3);
   DO_TEST (decompress_single_bit_error, XD3_ADLER32, 3);
 
-  IF_LZMA (DO_TEST (decompress_single_bit_error, XD3_SEC_LZMA, 3));
+  IF_LZMA (DO_TEST (decompress_single_bit_error, XD3_SEC_LZMA, 54));
   IF_FGK (DO_TEST (decompress_single_bit_error, XD3_SEC_FGK, 3));
   IF_DJW (DO_TEST (decompress_single_bit_error, XD3_SEC_DJW, 8));
 
@@ -2843,6 +2863,7 @@ xd3_selftest (void)
   IF_FGK (DO_TEST (secondary_fgk, 0, 1));
 
   DO_TEST (compressed_stream_overflow, 0, 0);
+  IF_LZMA (DO_TEST (compressed_stream_overflow, XD3_SEC_LZMA, 0));
 
 failure:
   test_cleanup ();
