@@ -1,6 +1,6 @@
 /* xdelta 3 - delta compression tools and library
  * Copyright (C) 2001, 2003, 2004, 2005, 2006, 2007,
- * 2008, 2009, 2010.  Joshua P. MacDonald
+ * 2008, 2009, 2010, 2011, 2012, 2013.  Joshua P. MacDonald
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -110,7 +110,6 @@
  */
 #ifndef _WIN32
 #include <stdint.h>
-typedef unsigned int usize_t;
 #else
 #define WIN32_LEAN_AND_MEAN
 #if XD3_USE_LARGEFILE64
@@ -125,7 +124,6 @@ typedef unsigned int usize_t;
 #define _WIN32_WINNT	0x0400
 #endif
 #include <windows.h>
-typedef unsigned int   usize_t;
 #ifdef _MSC_VER
 #define inline
 typedef signed int     ssize_t;
@@ -144,9 +142,8 @@ typedef ULONGLONG      uint64_t;
 #endif
 #endif
 
-/* TODO: note that SIZEOF_USIZE_T is never set to 8, although it should be for
- * a 64bit platform.  OTOH, may be that using 32bits is appropriate even on a
- * 64bit platform because we allocate large arrays of these values. */
+typedef uint32_t usize_t;
+
 #if XD3_USE_LARGEFILE64
 #define __USE_FILE_OFFSET64 1 /* GLIBC: for 64bit fileops, ... ? */
 #ifndef _LARGEFILE_SOURCE
@@ -160,7 +157,11 @@ typedef uint64_t xoff_t;
 #define SIZEOF_XOFF_T 8
 #define SIZEOF_USIZE_T 4
 #ifndef WIN32
+#if SIZEOF_SIZE_T == 8
+#define Q "z"
+#else
 #define Q "ll"
+#endif
 #else
 #define Q "I64"
 #endif
@@ -280,8 +281,8 @@ typedef struct _xd3_wininfo            xd3_wininfo;
  * provided as NULL, the stream returns XD3_GETSRCBLK. */
 
 typedef void*  (xd3_alloc_func)    (void       *opaque,
-				    usize_t      items,
-				    usize_t      size);
+				    size_t      items,
+				    usize_t     size);
 typedef void   (xd3_free_func)     (void       *opaque,
 				    void       *address);
 
@@ -692,10 +693,6 @@ struct _xd3_config
   usize_t             iopt_size;     /* entries in the
 					instruction-optimizing
 					buffer */
-  usize_t             srcwin_maxsz;  /* srcwin_size grows by a factor
-					of 2 when no matches are
-					found.  encoder will not seek
-				        back further than this. */
 
   xd3_getblk_func   *getblk;        /* The three callbacks. */
   xd3_alloc_func    *alloc;
@@ -729,6 +726,7 @@ struct _xd3_source
   const char         *name;          /* its name, for debug/print
 					purposes */
   void               *ioh;           /* opaque handle */
+  xoff_t              max_winsize;   /* maximum visible buffer */
 
   /* getblk sets */
   xoff_t              curblkno;      /* current block number: client
@@ -798,7 +796,6 @@ struct _xd3_stream
 					 size mask */
   usize_t           iopt_size;
   usize_t           iopt_unlimited;
-  usize_t           srcwin_maxsz;
 
   /* general configuration */
   xd3_getblk_func  *getblk;           /* set nxtblk, nxtblkno to scanblkno */
@@ -840,7 +837,7 @@ struct _xd3_stream
 				       * if there is at least one
 				       * match in the buffer. */
 
-  /* SRCWIN: these variables plus srcwin_maxsz above (set by config) */
+  /* SRCWIN */
   int                srcwin_decided;    /* boolean: true if srclen and
 					   srcbase have been
 					   decided. */
@@ -1070,11 +1067,11 @@ int     xd3_decode_memory (const uint8_t *input,
  *       src.curblkno = 0;
  *       src.onblk = source_size;
  *       src.curblk = source;
+ *       src.max_winsize = source_size;
  *       xd3_set_source(&stream, &src);
  *     }
  *
  *   config.flags = flags;
- *   config.srcwin_maxsz = source_size;
  *   config.winsize = input_size;
  *
  *   ... set smatcher, appheader, encoding-table, compression-level, etc.
